// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <util/message.h>
#include <gui/window.h>
#include <storage/fsnode.h>
 
#include "browsertab.h"
#include "messages.h"

using namespace os;
using namespace std;

bool BrowserTab :: m_ShowStatusBar = true;

BrowserTab :: BrowserTab( const Rect& cFrame, const String& cPath, uint32 nResizeMask, uint32 nFlags ) : View( cFrame, "browser", nResizeMask, nFlags )
{
	m_Browser = new EIconDirectoryView( Rect(), cPath, os::CF_FOLLOW_ALL );
	m_Browser->SetDirChangeMsg( new Message( MSG_BROWSER_CHANGE_DIR ) );
	m_Browser->SetSelChangeMsg( new Message( MSG_BROWSER_SELECTION_CHANGED ) );
	m_Browser->SetStatusBarChangeMsg( new Message( MSG_BROWSER_UPDATE_STATUSBAR ) );
	m_Browser->SetDirectoryLocked( true );
	m_Browser->SetAutoSort( true );
	AddChild( m_Browser );

	m_StatusBar = new StatusBar( Rect(), "status", CF_FOLLOW_LEFT | CF_FOLLOW_RIGHT | CF_FOLLOW_BOTTOM );
	m_StatusBar->AddPanel( "general_messages", "" );
	AddChild( m_StatusBar );

	Layout( true );
}

BrowserTab :: ~BrowserTab()
{
	delete m_Browser;
	delete m_StatusBar;
}

void BrowserTab :: SetFrame( const Rect& cRect, bool bNotifyServer )
{
	View::SetFrame( cRect, bNotifyServer );

	Layout();
}

void BrowserTab :: Paint( const Rect& cUpdateRect )
{
	// Erase background noise
	FillRect( cUpdateRect, get_default_color( COL_NORMAL ) );

	View::Paint( cUpdateRect );
}

void BrowserTab :: Layout( bool first )
{
	const float offset_x = 2.0f;
	const float offset_y = 2.0f;

	Rect cFrame = GetBounds();

	if( m_ShowStatusBar )
	{
		Point cStatusbar = m_StatusBar->GetPreferredSize( false );
		Rect cStatusFrame( offset_x, cFrame.bottom - offset_y - cStatusbar.y, cFrame.right - offset_x, cFrame.bottom - offset_y );
		m_StatusBar->SetFrame( cStatusFrame );

		Rect cBrowserFrame( offset_x, cFrame.top + offset_y, cFrame.right - offset_x, cStatusFrame.top - offset_y );
		m_Browser->SetFrame( cBrowserFrame );
	}
	else
	{
		m_StatusBar->SetFrame( Rect( 0, 0, 0, 0 ) );
		Rect cBrowserFrame( offset_x, cFrame.top + offset_y, cFrame.right - offset_x, cFrame.bottom - offset_y );
		m_Browser->SetFrame( cBrowserFrame );
	}

	m_Browser->SetTarget( this );
	if( !first )
		m_Browser->ReRead();
}

String BrowserTab :: GetPath()
{
	return m_Browser->GetPath();
}

void BrowserTab :: SetPath( String path, bool save_path )
{
	// We get two M_CHANGE_DIR messages. First when the new directory is opened and
	// then because of our SetPath() call. We ignore the second case here
	if( path == m_Browser->GetPath() )
		return;

	/// Remeber the previous path
	if( save_path )
		m_PrevoiusPaths.push( m_Browser->GetPath() );

	// Change the path
	m_Browser->SetPath( path );
	m_Browser->Clear();
	m_Browser->ReRead();
}

void BrowserTab :: SelectionChanged( void )
{
	int no_selected = 0;
	int no_selected_folders = 0;
	int no_folders = 0;
	uint32 size = 0;
	uint32 selected_size = 0;

	// Get number of selected icons and calculate some statistics for the directory
	for( uint i = 0 ; i < m_Browser->GetIconCount() ; i ++ )
	{	
		DirectoryIconData* data = static_cast< DirectoryIconData* >( m_Browser->GetIconData( i ) );
		String path( m_Browser->GetPath() + "/" + data->m_zPath );
		FSNode fs( path );

		if( fs.IsFile() )
			size += (uint) fs.GetSize();

		if( fs.IsDir() )
			no_folders++;

		if( m_Browser->GetIconSelected( i ) )
		{
			no_selected++;

			if( fs.IsDir() )
				no_selected_folders++;
			
			if( fs.IsFile() )
				selected_size += fs.GetSize();
		}
	}

	// No icons selected
	if( m_Browser->GetIconCount() == 0 )
	{
		m_StatusBar->SetText( "general_messages", "No objects founds." );
	}
	if( no_selected == 0 )
	{
		char buffer[ 1024 ];		
		sprintf(buffer, "Total %d objects (of which %d is folders). Total size %d bytes.", m_Browser->GetIconCount(), no_folders, size );

		m_StatusBar->SetText( "general_messages", buffer );
	}
	else
	{
		char buffer[ 1024 ];		
		sprintf(buffer, "%d objects (of which %d is folders) selected. Total size %d bytes.", no_selected, no_selected_folders, selected_size );

		m_StatusBar->SetText( "general_messages", buffer );
	}
}

void BrowserTab :: Back()
{
	if( m_PrevoiusPaths.empty() )
		return;

	String path = m_PrevoiusPaths.top();
	m_PrevoiusPaths.pop();

	SetPath( path, false );
}

void BrowserTab :: SetView( EIconDirectoryView::view_type type )
{
	if( m_Browser )
		m_Browser->SetView( type );
}


void BrowserTab :: HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{	
		case MSG_BROWSER_CHANGE_DIR:
		{
			String path;
			if( pcMessage->FindString( "file/path", &path.str() ) == 0 )
				SetPath( path );

			// Make sure that the window also gets the message
			GetWindow()->HandleMessage( pcMessage );
			break;
		}
		case MSG_BROWSER_SELECTION_CHANGED:
			SelectionChanged();		
			break;
	}
}

vector<String> BrowserTab :: GetSelectedFiles()
{
	vector<String> list_of_selected_files;

	// Get number of selected icons and calculate some statistics for the directory
	for( uint i = 0 ; i < m_Browser->GetIconCount() ; i ++ )
	{	
		if( m_Browser->GetIconSelected( i ) )
		{
			DirectoryIconData* data = static_cast< DirectoryIconData* >( m_Browser->GetIconData( i ) );
			list_of_selected_files.push_back( String( GetPath() + "/" + data->m_zPath ) );
			
		}
	}

	return list_of_selected_files;
}

